#ifndef _DLIMAGERESIZE_H_
#define _DLIMAGERESIZE_H_

#include "DLImage.h"
#include "DLConst.h"
#include "DLTypes.h"

#include <math.h>
#include <stdio.h>
#include <malloc.h>

using namespace std;

/**
* \brief The DLImageResize class computes a rescaled representation of an input image based on
* the specified factor and return a crisp rescaled image in grayscale format using the Lanczos filter.
* 
* The algorithm works best for native grayscale images since it makes use of grayscale information.
* Binary and color images are converted to grayscale images first before processing.
* It well preserves fine details on the image, especially on edges and contours.
*/


class DLImageResize
{
public:

	/**
	* Default DLImageResize Constructor
	*/
	DLImageResize() {};

	/**
	* Default DLImageResize Destructor
	*/
	virtual ~DLImageResize() {};

	/**
	* Image rescaling function that creates a crisp resized grayscale image using Lanczos filter of given support size
	* \param srcImage input DLImage
	* \param rescale_ratio rescale factor (should be larger than 1 if you want to shrink an image) in double
	* \param fwidth filter width (the default support size is choosen as 3) in double
	* \return the rescale image as DLImage
	*/
	DLImage dlCreateResizedImage(const DLImage & srcImage, double rescale_ratio, double fwidth=3.0);

private:

	/**
	* Copy a given column of image data and hold it in buffer
	* \param DL_BYTE *column, DL_BYTE* to buffer for holding the copyed column
	* \param DL_BYTE *imageData, DL_BYTE* source grayscale image data
	* \param int x, the column number
	* \param int imageWidth, the image width
	* \param int imageHeight, the image height
	*/
	void dlGetImageColumn(DL_BYTE *column, DL_BYTE *imageData, int x, int imageWidth, int imageHeight);

	/**
	* Copy a given row of image data and hold it in buffer
	* \param DL_BYTE *row, DL_BYTE* to buffer for holding the copyed row
	* \param DL_BYTE *imageData, DL_BYTE* source grayscale image data
	* \param int y, the row number
	* \param int imageWidth, the image width
	* \param int imageHeight, the image height
	*/
	void dlGetImageRow(DL_BYTE *row, DL_BYTE *imageData, int y, int imageWidth, int imageHeight);

	
};//end class

#endif //_DLIMAGERESIZE_H_
