#ifndef _DLTESTMANAGER_H
#define _DLTESTMANAGER_H

#include "DLTest.h"
#include "DLConst.h"

#include <string>
#include <vector>
#include <iostream>

using namespace std;

const static char DEFAULT_LOG_FILE[] = "Test.log";

/**
* \brief The DLTestManager class manages the run time environment for the set of DLTests defined by the developer.
* Upon successful completion of each DLTest, it displays the DLTest log messages to the screen. When any 
* run-time exception occurs, it displays the detailed error message to the screen. The developer can make use
* of this error catching facility for testing and debugging purposes. 
* 
* All the run-time exception types defined in DLException class are supported. 
* DLTestManager does not interpret, but simply make the command line arguments availabe to each DLTest.  
*
*/
class DLTestManager
{
public:

	/**
    * Default DLTestManager Constructor. Initialize the list of command line arguments
	* \param argc An integer that contains the count of arguments that follow in argv. 
	* The argc parameter is always greater than or equal to 1. 
	* \param argv An array of null-terminated strings representing command-line arguments
	* entered by the user of the program. By convention, argv[0] is the command with which the
	* program is invoked, argv[1] is the first command-line argument, and so on, until argv[argc],
	* which is always NULL.
    */
	DLTestManager(int argc, char** argv) { 	dlArgc = argc; dlArgv = argv; }

	/**
    * Default DLTestManager Destructor
    */
	~DLTestManager() { }

	/**
    * Add a new DLTest to the test sequence
    * \param newTest DLTest* to the added test 
    */
	void addTestCase(DLTest * newTest);

	/**
    * Run the sequence of DLTests and catch any run-time exceptions
     * \return 0 if all the DLTests completed successfully
    */
	int go();

private:

	/// The number of arguments that follow in dlArgv
	int dlArgc;

	/// The array of null-terminated strings representing command-line arguments
	char** dlArgv;

	/// The set of developer defined DLTests
	vector <DLTest*> dlTests;

};//end class

#endif // _DLTESTMANAGER_H
