#include "xml.h"

xml::xml(void)
{
	strXMLDoc = "";
	curStringIdx = -1;
}

xml::~xml(void)
{
}

//Add, Insert an Element and its Data, Get a element with all sub elements
bool xml::InsertSiblingElem(string strElemName, string strData)
{
	//insert begining
	string strInsert = "\n";

	strInsert = "<"; 
	strInsert += strElemName;
	strInsert += ">";
	strInsert += "\n";

	//insert data
	strInsert += strData;

	//insert end
	strInsert += "</";
	strInsert += strElemName;
	strInsert += ">";

	if( -1 == curStringIdx) // outter most parent
	{
		curStringIdx = 0;
		strXMLDoc.empty();
		strXMLDoc.insert( curStringIdx, strInsert );
	}
	else
	{
		if( 0 == curStringIdx)
		{
			return false;
		}
		else
		{
			// insert as sibling
			string strElemName = GetElemName( curStringIdx );
			size_t iInsertPos = GetEndElemPos( curStringIdx ) + 1 + strElemName.size() + 1 + 1;
			strXMLDoc.insert( iInsertPos, strInsert );

			curStringIdx = iInsertPos;
		}
	}

	return true;
}

bool xml::InsertChildElem(string strElemName, int level, string strData)
{
	//insert begining
	string strInsert = "\n";

	for(int i = 1; i < level; i++)
		strInsert += "\t";

	strInsert += "<";
	strInsert += strElemName;
	strInsert += ">";
	strInsert += "\n";

	//insert data
	strInsert += strData;

	for(int i = 1; i < level; i++)
		strInsert += "\t";
	//insert end
	strInsert += "</";
	strInsert += strElemName;
	strInsert += ">";

	size_t iInsertPos = GetPrevRightBracketPos( GetEndElemPos(curStringIdx) ) + 1;//insert to the buttom
	strXMLDoc.insert( iInsertPos, strInsert );
	
	return true;
}

string xml::GetCurElemDoc()
{
	return GetElemDoc( curStringIdx );
}

string xml::GetElemDoc( size_t iPos )
{
	string strSubDoc;
	string strElemName;

	size_t iCur = curStringIdx;
	curStringIdx = iPos;

	char chCurChar = strXMLDoc.at( iPos );
	
	if( '<' != chCurChar )
	{
		strSubDoc = "";
	}
	else
	{
		//find >, get current elem name
		strElemName = GetCurElemName();

		//find </elem name>
		size_t iEnd = GetEndElemPos(iPos) + 1 + strElemName.size() + 1;

		//return strings between them.
		strSubDoc = strXMLDoc.substr( iPos,iEnd - iPos + 1 );

	}

	curStringIdx = iCur;
	return strSubDoc;
}


string xml::GetCurElemName()
{
	return GetElemName( curStringIdx );
}


string xml::GetElemName( size_t iPos )
{
	char chCurChar = strXMLDoc.at( iPos );

	if( '<' != chCurChar )
		return "";

	size_t iSpacePos = GetNextSpacePos(iPos);
	size_t iRightBracketPos = GetNextRightBracketPos(iPos);

	size_t iEndPos = min(iSpacePos,iRightBracketPos);
	
	string strElemName = strXMLDoc.substr( iPos + 1, (iEndPos - 1) - (iPos + 1) + 1 );
	return strElemName;
}

string xml::GetCurElemData()
{
	return GetElemData( curStringIdx );
}

string xml::GetElemData( size_t iPos )
{
	size_t iRightEnd = GetEndElemPos(iPos) - 1;
	size_t iLeftEnd = GetPrevRightBracketPos( iRightEnd ) + 1;

	string strData;

	if( iLeftEnd > iRightEnd )
		strData = "";
	else
		strData = strXMLDoc.substr( iLeftEnd, iRightEnd - iLeftEnd + 1 );

	return strData;
}

bool xml::DelCurElem( bool bRecursive )
{
	if ( bHasChild() &&  0 == bRecursive )
		return false;

	string strElemName = GetCurElemName();
	size_t iEndPos = GetEndElemPos() + 1 + strElemName.size() + 1;

	strXMLDoc.erase(curStringIdx, iEndPos - curStringIdx + 1);

	//some further work of adjusting curStringIdx.
	size_t iNewPos = GetPrevLeftBracketPos( curStringIdx );
	
	if( '/' == strXMLDoc.at( iNewPos + 1 ) )
	{
		//go to parent pos first
		int iStack = 1;
		size_t iPos1 = iNewPos;
		size_t iOldPos;

		while( npos != ( iPos1 = GetPrevLeftBracketPos(iPos1) ) )
		{
			if( '/' == strXMLDoc.at(iNewPos + 1) )	
				iStack +=1;
			else
				iStack -=1;

			iOldPos = iPos1;

			if( 0 == iStack )
				break;
		}

		if( bHasChild() )
			Go2ChildLevel();
		else
			curStringIdx = iOldPos;
	}
	else
	{
		;
	}

	return true;
}


// Add, Set, Delete attributes in Currrent Element
bool xml::AddDynamicAttrib( map <string, string> tagMap )
{
	map <string, string>::const_iterator itr;

	for(itr = tagMap.begin(); itr != tagMap.end(); ++itr)
	{
		bool bret = AddAttrib( (*itr).first,  (*itr).second );
		if( false == bret )
			return false;
	}
	
	return true;
}

bool xml::AddAttrib(string strAttrib, int nValue)
{
	char szVal[25];
	sprintf( szVal, "%d", nValue );
	return AddAttrib( strAttrib,  szVal );
}

bool xml::AddAttrib(string strAttrib, string strValue)
{
	string strAtt = strAttrib;
	strAtt += "=\"" ;
	strAtt += strValue;
	strAtt += "\"";

	size_t iSpacePos = GetNextSpacePos( curStringIdx );
	size_t iRightBracketPos = GetNextRightBracketPos( curStringIdx );

	size_t iInsertPos = min(iSpacePos,iRightBracketPos);

	string strtmp = " ";
	strtmp += strAtt;
 	strAtt = strtmp;

	strXMLDoc.insert( iInsertPos, strAtt );

	return true;
}

bool xml::SetAttrib(string strAttrib, int nValue)
{
	char szVal[25];
	sprintf( szVal, "%d", nValue );
	return SetAttrib( strAttrib, szVal );
}

void xml::setXML( string newXML )
{
	strXMLDoc = newXML;
	curStringIdx = 0;
}

bool xml::SetAttrib(string strAttrib, string strValue)
{
	if (false == DelAttrib(strAttrib))
		return false;
	
	return AddAttrib(strAttrib,strValue);;
}

bool xml::DelAttrib(string strAttrib)
{
	size_t iEnd = GetNextRightBracketPos( curStringIdx );
	size_t iAttrPos = strXMLDoc.find( strAttrib, curStringIdx);

	if( ( npos == iAttrPos ) || ( iEnd < iAttrPos ) )
		return false;

	size_t iFirstQuote = GetNextQuotePos(iAttrPos);
	size_t iSecondQuote = GetNextQuotePos(iFirstQuote );

	strXMLDoc.erase(iAttrPos, iSecondQuote - iAttrPos + 1 );	
	
	if( ' ' == strXMLDoc.at(iAttrPos - 1) )
		strXMLDoc.erase(iAttrPos - 1, 1);

	return true;
}

string xml::GetAttrib(string strAttrib)
{
	size_t iEnd = GetNextRightBracketPos( curStringIdx );
	size_t iAttrPos = strXMLDoc.find( strAttrib, curStringIdx);

	if( ( npos == iAttrPos ) || ( iEnd < iAttrPos ) )
		return "";

	size_t iFirstQuote = GetNextQuotePos(iAttrPos);
	size_t iSecondQuote = GetNextQuotePos(iFirstQuote);

	string strattri = strXMLDoc.substr(iFirstQuote + 1, iSecondQuote - iFirstQuote - 1 );
	return strattri;
}

map <string, string> xml::GetDynamicAttrib( int num )
{
	map <string, string> mapRet;
    size_t iEnd = GetNextRightBracketPos( curStringIdx );
	size_t iEq = curStringIdx;

	for (size_t i = 0; i < num; i++ )
		iEq = GetNextEqualSignPos( iEq );

	while( iEq < iEnd )
	{
		string strAttrib = GetAttribName( iEq );
		string strAttrbVal = GetAttrib( strAttrib+"=" );
		mapRet[ strAttrib ] = strAttrbVal;

		iEq = GetNextEqualSignPos( GetNextQuotePos( GetNextQuotePos( iEq ) ) );
	}

	return mapRet;
}

string xml::GetAttribName( size_t iEq )
{
	string strAttrib;
	size_t iAttrEnd = iEq;

	while( ' ' == strXMLDoc.at(iAttrEnd) )
		iAttrEnd --;

	size_t iAttrStart = GetPrevSpacePos(iAttrEnd) + 1;
	
	strAttrib = strXMLDoc.substr(iAttrStart, iAttrEnd - iAttrStart );	


	return strAttrib;
}


// Navigation
bool xml::bHasChild()
{
	size_t iNextEnd = GetNextEndBracketPos( curStringIdx );
	size_t iEndElem = GetEndElemPos();

	if( iNextEnd == iEndElem)
		return false;
	else
		return true;
}


bool xml::Go2ChildLevel()
{
	if( !bHasChild() )
	{
		return false;
	}

	curStringIdx = GetNextLeftBracketPos(curStringIdx);
	return true;
}

bool xml::Go2ParentLevel()
{
	if( 1 == curStringIdx )
		return false;

	size_t iInitialPos = curStringIdx;
	string str = GetCurElemName();
	bool bStop = false;
	
	while ( !bStop )
	{
		size_t iPrevStart = GetPrevLeftBracketPos( iInitialPos );
		
		if( '/' == strXMLDoc.at(iPrevStart + 1))
		{
			iPrevStart = GetPrevLeftBracketPos( iPrevStart );
		}

		size_t iEndPos = GetEndElemPos(iPrevStart);
	
		if( iInitialPos >  iEndPos)
		{
			iInitialPos = iPrevStart;
		}
		else
		{
			curStringIdx = iPrevStart;
			bStop = true;
		}
	}

	return true;
}

bool xml::GoPrevSibling()
{
	size_t iPrevStart = GetPrevLeftBracketPos( curStringIdx );

	if( npos == iPrevStart )
		return false;
		
	if( '/' == strXMLDoc.at(iPrevStart + 1))
	{
		iPrevStart = GetPrevLeftBracketPos( iPrevStart );
		curStringIdx = iPrevStart;
		return true;
	}
	else
	{
		return false;
	}
}

bool xml::GoNextSibling()
{
	size_t iCurEnd = GetEndElemPos(curStringIdx);
	size_t iNextPossibleElem = GetNextLeftBracketPos(iCurEnd);

	if( npos == iNextPossibleElem )
		return false;

	if( '/' == strXMLDoc.at(iNextPossibleElem + 1) )
		return false;
	else
	{
		curStringIdx = iNextPossibleElem;
		return true;
	}
}

// File io
bool xml::LoadXml(string strFileName)
{
	FILE* fp = fopen( strFileName.c_str(), "rb" );
	
	if ( ! fp )
	{
		return false;
	}

	// File is open
	fseek( fp, 0, SEEK_END );
	size_t nFileByteLen = ftell(fp);
	fseek( fp, 0, SEEK_SET );
	strXMLDoc.erase();

	// Read file
	if ( nFileByteLen )
	{
		char* pszBuffer = new char[nFileByteLen];
		fread( pszBuffer, nFileByteLen, 1, fp );
		strXMLDoc.assign( pszBuffer, nFileByteLen );
		delete [] pszBuffer;
	}

	fclose( fp );
	
	curStringIdx = 0;

	return true;
}

bool xml::SaveXml(string strFileName)
{
	bool bSuccess = true;
	
	FILE* fp = fopen( strFileName.c_str(), "wb" );
	if ( ! fp )
	{
		return false;
	}

	/* Insert copyright information of GEDI into xml file */
	string GEDIHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	GEDIHeader += "<!--GEDI is developed at Language and Media Processing Laboratory, University of Maryland.-->\n";
	GEDIHeader += "<GEDI xmlns=\"http://lamp.cfar.umd.edu/GEDI\" version=\"1.0\">\n";
	string GEDIFooter = "\n</GEDI>";
	
	strXMLDoc = GEDIHeader + strXMLDoc;
	strXMLDoc = strXMLDoc + GEDIFooter;
    
	// File is open
	size_t nLength = (size_t)strXMLDoc.size();

	if ( nLength )
		bSuccess = ( fwrite( strXMLDoc.c_str(), nLength, 1, fp ) == 1 );
	
	fclose(fp);

	return bSuccess;
}

// Positions
size_t xml::GetEndElemPos()
{
	return GetEndElemPos( curStringIdx );
}

size_t xml::GetEndElemPos( size_t iPos )
{
	int iStack = 1;
	size_t iNewPos = iPos;
	size_t iOldPos;

	if( npos == GetNextLeftBracketPos(iPos) )
		return npos;

	while( npos != ( iNewPos = GetNextLeftBracketPos(iNewPos) ) )
	{
		if( '/' == strXMLDoc.at(iNewPos + 1) )	
			iStack -=1;
		else
			iStack +=1;

		iOldPos = iNewPos;

		if( 0 == iStack )
			break;
	}

	return iOldPos;
}


size_t xml::GetPrevLeftBracketPos(size_t iPos)
{
	return strXMLDoc.rfind( "<", iPos - 1);
}

size_t xml::GetNextLeftBracketPos(size_t iPos)
{
	return strXMLDoc.find( "<", iPos + 1);
}

size_t xml::GetPrevRightBracketPos(size_t iPos)
{
	return strXMLDoc.rfind( ">", iPos - 1);
}

size_t xml::GetNextRightBracketPos(size_t iPos)
{
	return strXMLDoc.find( ">", iPos + 1);
}

size_t xml::GetNextEqualSignPos(size_t iPos)
{
	return strXMLDoc.find( "=", iPos + 1);
}

size_t xml::GetPrevEndBracketPos(size_t iPos)
{
	return strXMLDoc.find( "</", iPos - 1);
}

size_t xml::GetNextEndBracketPos(size_t iPos)
{
	return strXMLDoc.find( "</", iPos + 1);
}

size_t xml::GetNextSpacePos(size_t iPos)
{
	return strXMLDoc.find( " ", iPos + 1);
}


size_t xml::GetPrevQuotePos(size_t iPos)
{
	return strXMLDoc.rfind( "\"", iPos - 1);
}

size_t xml::GetNextQuotePos(size_t iPos)
{
	return strXMLDoc.find( "\"", iPos + 1);
}

size_t xml::GetPrevSpacePos(size_t iPos)
{
	return strXMLDoc.rfind( " ", iPos - 1);
}

string xml::RetrieveSpecifiedElem( string strElemName )
{
	size_t pos = strXMLDoc.find( strElemName, 0 );
	pos = GetPrevLeftBracketPos( pos );
	return GetElemDoc( pos );
}