#include "DLImage.h"
#include "DLDocument.h"
#include "DLPage.h"
#include "DLZone.h"
#include "DLTagList.h"
#include "DLLogoDetect.h"
#include "DLLogoDetectBatchTest.h"
#include "DLTestManager.h"
#include "DLConst.h"
#include "DLxml.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

#define MAX_PATH 260
#define DEFAULT_DETECTION_THRE 700

DLLogoDetectBatchTest::DLLogoDetectBatchTest(char* name, int argc, char** argv) : DLTest(name, argc, argv)
{
	testName = name;
	dlArgc = argc;
	dlArgv = argv;
}

DLLogoDetectBatchTest::~DLLogoDetectBatchTest(void)
{
}

void DLLogoDetectBatchTest::showHelp()
{
	/* Show command line help information */
	printf( "LogoDetect [-x] [-e] [-t DetectionThreshold] [-l] filename\n" );
	printf( "     -x  ---  Write detection results into an xml file for each input document.\n" );
	printf( "              It creates a new xml file if no associated xml file exists.\n" );
	printf( "     -e  ---  Erase all previous logo detection results stored in xml files.\n" );
	printf( "              Write new results to xml file.\n" );
	printf( "     -t  ---  Specify your detection threshold. The default is 700.\n" );
	printf( "     -l  ---  File containing the list of input images to execute\n" );
	printf( "     -h  ---  Show help at command line\n" );
}

int DLLogoDetectBatchTest::go()
{
	char inputFile[MAX_PATH];
	char logFile[MAX_PATH];
	bool bFileList = false;
	bool bOutputXML = false;
	bool bErasePrevious = false;
	unsigned long detectionThreshold = DEFAULT_DETECTION_THRE;

	/* Parse command line arguments */
	if(dlArgc == 1)
	{
		showHelp();
		return 0;
	}
    
	for(int i=1; i<dlArgc; i++)
	{
		if(dlArgv[i][0] != '-') break;
		++i;
		switch(tolower(dlArgv[i-1][1]))
		{
			case 'x':
				bOutputXML = true;
				i--;
				break;
			case 'e':
				bErasePrevious = true;
				i--;
				break;
			case 't':
				detectionThreshold = atol( dlArgv[i] );
				break;
			case 'l':
				bFileList = true;
				i--;
				break;
			case 'o':
				strcpy( logFile, dlArgv[i] );
				break;
			case '?':
			case 'h':
				showHelp();
				return 0;
			default:
				printf( "Unable to interprete your command!\n" );
				showHelp();
				return -1;
		}
	}

	/* Show commmand line help panel if incorrect arguments are supplied */
	if( i != dlArgc-1 )
	{
		showHelp();
		return -1;
	}

	/* Grab the input file name specified at the command line */
	strcpy( inputFile, dlArgv[i] );

	vector <string> imageFileList;
	string imageFileName;

	/* Read in the list of images for processing if the list argument in the command line is supplied */
	if(bFileList)
	{
		ifstream infile(inputFile);
		cout << "Batch file - " << string(inputFile) << endl << endl;
		appendToTestLog("Batch file - " + string(inputFile) + "\n");
		if(infile)
		{
			while (!infile.eof())
			{
				imageFileName = "";
				infile >> imageFileName;
				if(imageFileName != "\n" && imageFileName != "")
					imageFileList.push_back(imageFileName);
			}
		}
		infile.close();
	}
	/* Process the single file if the list argument in the command line is empty */
	else
	{
		imageFileList.push_back(static_cast<string>(inputFile));
		cout << "Image file - " << string(inputFile) << endl;
		appendToTestLog("Image file - " + string(inputFile) + "\n");
	}

	int NrOfPreDetectedLogos = 0;
	int TotalDetectedLogos = 0;
	char docCounter[5], pageCounter[5], zoneCounter[5];
	int zoneNumber = 0;

	list <DLLogoCandidate> detectedLogoList;
	list <DLLogoCandidate>::iterator logoCandidate_Iter;
	DLDocument::DLPagePtrIterator page_Iter;
	string srcImageFullPathName;

	/* Detect logos from input document images and store them */
	for(int imageIndex = 0; imageIndex < int(imageFileList.size()); imageIndex++)
	{
		srcImageFullPathName = (string)imageFileList[imageIndex];
		DLImage docImage(srcImageFullPathName.c_str());
		DLDocument currentDoc;

		sprintf(docCounter, "%d", imageIndex+1);
		sprintf(pageCounter, "%d", 1);
		string docID = "Document" + static_cast<string>(docCounter);
		string pageID = "Page" + static_cast<string>(pageCounter);

		NrOfPreDetectedLogos = 0;
		bool xmlFileFound = false;
		int fileExtPosition = int(srcImageFullPathName.find("."));
		int dirSepPosition = srcImageFullPathName.find_last_of(static_cast<string>(DL_DIR_SEPARATOR), fileExtPosition);
		string srcImageFileName, xmlFileName, xmlFilePath, xmlFullPathName;

		if(dirSepPosition != -1)	
		{
			srcImageFileName = srcImageFullPathName.substr(dirSepPosition);
			xmlFileName = srcImageFullPathName.substr(dirSepPosition, fileExtPosition);
			xmlFilePath = srcImageFullPathName.substr(0, dirSepPosition);
		}
		else 
		{
			srcImageFileName = srcImageFullPathName;
			xmlFileName = srcImageFullPathName.substr(0, fileExtPosition);
			xmlFilePath = "";
		}
		xmlFileName = xmlFileName + ".xml";
		xmlFullPathName = xmlFilePath + xmlFileName;

		ifstream xmlfile(xmlFileName.c_str());
		if(xmlfile)
		{
			/* Read in the xml file associated with the input document image if the xml file exists */
			xmlFileFound = true;
			cout << "Reading xml file - " << string(xmlFileName) << endl << endl;	
			DLxml xmlInput;
			currentDoc = xmlInput.dlReadFromXmlFile( xmlFileName, xmlFilePath );
			page_Iter = currentDoc.begin();

			if((*page_Iter)->dlHasZones())
			{
				/* iterate over all the zones and erase previous detection results */
				DLPage::DLZonePtrIterator zone_Iter1, zone_Iter2;
				for ( zone_Iter1 = (*page_Iter)->begin(); zone_Iter1 != (*page_Iter)->end(); )
				{
					DLTagList currentTagList = (*zone_Iter1)->dlGetAllTags();
					if(!currentTagList.dlIsTagListEmpty() && currentTagList.dlGetTag("gedi_type") == "DLDetectedLogo")
					{
						if(bErasePrevious) {
							zone_Iter2 = zone_Iter1;
							zone_Iter2++;
							(*page_Iter)->dlDeleteZone(*zone_Iter1);
							zone_Iter1 = zone_Iter2;
						}
						else {
							NrOfPreDetectedLogos++;
							zone_Iter1++;
						}
					}
					else {
						zone_Iter1++;
					}
				}
			}
		}
		else
		{
			/* Create a single page document for storing detection information if there is no associated xml file */
			DLDocument thisDoc(docImage, pageID, docID);
			currentDoc = thisDoc;
			page_Iter = currentDoc.begin();
		}
		xmlfile.close();

		cout << "Processing " << srcImageFileName.c_str() << " ..." << endl;
		DLLogoDetect logo(srcImageFullPathName.c_str(), detectionThreshold);
		list <DLLogoCandidate> docLogoList = logo.dlGetDetectedLogos();

		if (!docLogoList.empty())
		{
			for ( logoCandidate_Iter = docLogoList.begin(), zoneNumber = NrOfPreDetectedLogos+1; logoCandidate_Iter != docLogoList.end(); logoCandidate_Iter++, zoneNumber++ )
			{
				sprintf(zoneCounter, "%d", zoneNumber);
				string zoneID = "DLDetectedZone" + static_cast<string>(zoneCounter);

				DLZone *logoZone = new DLZone();
				*logoZone = logoCandidate_Iter->logoRegion;
				logoZone->dlSetZoneID(zoneID);
				logoZone->dlSetTag("id", "None");
				logoZone->dlSetTag("gedi_type", "DLDetectedLogo");
				(*page_Iter)->dlAppendZone(logoZone);

				// save the detected logo regions
				int logoOrigX = logoZone->dlGetZoneOrigin().x;
				int logoOrigY = logoZone->dlGetZoneOrigin().y;
				int logoWidth = logoZone->dlGetZoneWidth();
				int logoHeight = logoZone->dlGetZoneHeight();
				string logoFileName = srcImageFullPathName.substr(0, srcImageFileName.find_last_of(".")) + "_logo.tif";
				//docImage.dlSubImage(logoOrigX, logoOrigY, logoWidth, logoHeight).dlSaveImage(logoFileName.c_str());

				detectedLogoList.push_back(*logoCandidate_Iter);
				TotalDetectedLogos++;
			}
		}

		/* Update existing xml file or create a new xml file containing the detection results */
		if(bOutputXML)
		{
			DLxml xmlOutput(currentDoc, srcImageFileName);	
			xmlOutput.dlSaveToXmlFile(xmlFullPathName);
			if(xmlFileFound) {
				cout << string(xmlFullPathName) << " updated" << endl << endl;
			}
			else {
				cout << string(xmlFullPathName) << " created" << endl << endl;
			}
		}
	}

	cout << TotalDetectedLogos << " logos detected from " << imageIndex << " documents" << endl;
	
	appendToTestLog("\n\n\n******************************************************\n");
	appendToTestLog("**Test Summary\n");
	appendToTestLog("******************************************************\n\n");

	return 0;

}



