#ifndef _DLSTAMPDETECTBATCHTEST_H
#define _DLSTAMPDETECTBATCHTEST_H

#include "DLTest.h"
#include "DLZone.h"
#include <string>

/**
 * \class DLStampDetectBatchTest
 *
 * \brief DLStampDetectBatchTest is a command line application that runs the stamp detector on a list of document
 * images specified by the user.
 *
 * Its command line syntax is:
 * <p>StampDetect [-x] [-e] [-min minArea] [-max maxArea] [-l] filename</p>
 *
 * <br>
 *  -x   ---  Write detection results into an xml file for each input document.
 *            It creates a new xml file if no associated xml file exists.
 *  -e   ---  Remove all previous stamp detection results stored in xml files.
 *            New results will be appended to previous results if this argument is not used.
 *  -min ---  Specify the minimum area of detected stamp in pixel squares.
 *            Default value of 50000 is used if this argument is ignored.
 *  -max ---  Specify the maximum area of detected stamp in pixel squares.
 *            Default value of 200000 is used if this argument is ignored.
 *  -l   ---  File containing the list of input images to execute
 *  -h   ---  Show help at command line<br>
 *
 * </p><br>
 */
class DLStampDetectBatchTest : public DLTest
{
public:

	/**
    * Default DLStampDetectBatchTest Constructor
	* \param argc An integer that contains the count of arguments that follow in argv. 
	* The argc parameter is always greater than or equal to 1. 
	* \param argv An array of null-terminated strings representing command-line arguments
	* entered by the user of the program. By convention, argv[0] is the command with which the
	* program is invoked, argv[1] is the first command-line argument, and so on, until argv[argc],
	* which is always NULL.
	* \param name A string given to the test.
    */
	DLStampDetectBatchTest(char* name, int argc, char** argv);
    
	/**
	* Default DLStampDetectBatchTest Destructor
	*/
	~DLStampDetectBatchTest(void);

	/**
	* The Main function of DLStampDetectBatchTest. Running stamp detector on a list of document images
	*/
	int go();

	/**
	* Show help information for running the stamp detection driver application
	*/
	void showHelp();

};

#endif
