#!/usr/bin/perl

# This script is used to compile, make, install, uninstall, and reconfigure the Doclib files.

# Last updated May 18, 2005.

# DEFINE AND CHECK REGULAR PARAMETERS
(defined($ENV{DOCLIB})) || die "ERROR: The environment variable \$DOCLIB is not defined. Please source the 'params' file from the tcsh or csh environment";
$DOCLIB = $ENV{DOCLIB};
(defined($ENV{OS})) || die "ERROR: The environment variable \$OS is not defined. Please source the 'params' file from the tcsh or csh environment";
$OS = $ENV{OS};
(defined($ENV{OSLIST})) || die "ERROR: The environment variable \$OSLIST is not defined. Please source the 'params' file from the tcsh or csh environment";
$OSLIST = $ENV{OSLIST};
@osList = split(/ /, $OSLIST);
(defined($ENV{CONF_DL})) || die "ERROR: The environment variable \$CONF_DL is not defined.  Please source the \"params\" file from the tcsh or csh environment";
$CONF_DL = $ENV{CONF_DL};
$numList = @osList;
$curDir = `pwd`;
chomp($curDir);
$self = &formatInput($0, $curDir);

# FIND FILES CONTAINING CONFIGURE FILES
@cDir = `find $DOCLIB -name 'configure' -print | sed 's|/configure\$||' | sort -f`;
  # directories containing configure files
$cCnt = @cDir;
  # number of directories containing configure files
$inOptDir[0] = "";
  # $inOptDir[1 through $cCnt] are the directories containing the configure files
$inOptDir[1] = "";
for ($ind = 1; $ind <= $cCnt; $ind++) {
  $inOptDir[$ind] = shift(@cDir);
  chomp($inOptDir[$ind]);

  # this is to ensure that the main doclib library is compiled first
  if ($inOptDir[$ind] =~ /doclib$/) {
    $tempVal = $inOptDir[1];
    $inOptDir[1] = $inOptDir[$ind];
    for ($ine = $ind; $ine > 2; $ine--) {
      $inOptDir[$ine] = $inOptDir[$ine-1];
    }
    $inOptDir[2] = $tempVal;
  }
}

# USAGE
$inOptList = "      0: ALL; process all the below listed directories under $DOCLIB/\n";
for ($ind = 1; $ind <= $cCnt; $ind++) {
  $inOptList = sprintf("%s      %d: %s\n", $inOptList, $ind, $inOptDir[$ind]);
}
$argCnt = @ARGV;
$usage =
  "This script is used to compile, make, install, uninstall, and reconfigure the Doclib files under $DOCLIB/.\n".
  "Usage: $self <input> <task>\n".
  "   <input>: 0-$cCnt; a number representing the directory/directories under $DOCLIB/ on which to process. Options are as follows:\n".
  "$inOptList".
  "   <task>: Various options for special use.\n".
  "      make: Perform a regular compilation.\n".
  "      restore: Restore directory to original untouched state.\n".
  "      clean: Same as \"restore\" but leaves files created by running \"save\" intact.  These files will have one of the string(s) \"$OSLIST\" as part of the file name.\n". 
  "      install: Install header files, libraries, and executables to locations specified in 'params' file.\n".
  "      uninstall: Uninstall header files, libraries, and executables from locations specified in 'params' file.\n".
  "      save: Saves libraries and executables locally (in the same directory as the source code) with '$OS' appended after the root name.  This is only useful when libraries and executables compiled from various operating systems need to be saved in the same directory.\n".
  "      reconfigure: Perform reconfiguration.  This must be done whenever Makefile.am or configure.in are changed.\n".
  "   A file called \"log.txt\" will be generated/appended summarizing the commands run to perform the chosen task(s).\n".
  "\n";
unless ($argCnt == 2) {
  die "$usage";
}

# READ AND CHECK INPUT
$in = $ARGV[0];
unless (($in =~ /^[0-9]+$/) && (0 <= $in) && ($in <= $cCnt)) {
  die "<input> '$in' must be a number between 0 and $cCnt inclusive.\n\n$usage";
}
$task = $ARGV[1];
if ($task =~ /^make$/) {
  $opt = 0;
} elsif ($task =~ /^restore$/) {
  $opt = 1;
} elsif ($task =~ /^clean$/) {
  $opt = 2;
} elsif ($task =~ /^install$/) {
  $opt = 3;
} elsif ($task =~ /^uninstall$/) {
  $opt = 6;
} elsif ($task =~ /^save$/) {
  $opt = 4;
} elsif ($task =~ /^reconfigure$/) {
  $opt = 5;
} else {
  die "<task> '$task' not recognized.\n\n$usage";
}

# PROCESS INPUT
open(LOG, ">> log.txt") || die "ERROR: Unable to open 'log.txt' for writing/appending";
$date = `date`;
chomp($date);
print LOG "Running '$self $in $task' from $curDir at $date.\n\n";
if ($in) {
  $start = $in;
  $end = $in;
} else {
  $start = 1;
  $end = $cCnt;
}
for ($ind = $start; $ind <= $end; $ind++) {
  print LOG "cd $inOptDir[$ind]\n";
  chdir("$inOptDir[$ind]") || die "ERROR: Unable to chdir to $inOptDir[$ind]";
  if ((0 <= $opt) && ($opt <= 4)) {
    print LOG "./configure $CONF_DL\n";
    system("./configure $CONF_DL");
  }
  if ($opt == 0) {
    # regular make
    print LOG "make\n";
    system("make");
  } elsif (($opt == 1) || ($opt == 2)) {
    # make clean and removal of unnecessary configure file by-products
    print LOG "make clean\n";
    system("make clean");
    if (-e "config.1og") {
      print LOG "rm -f config.log\n";
      unlink("config.log") || die "ERROR: Unable to unlink (delete) $inOptDir[$ind]/config.1og";
    }
    if (-e "config.status") {
      print LOG "rm -f config.status\n";
      unlink("config.status") || die "ERROR: Unable to unlink (delete) $inOptDir[$ind]/config.status";
    }
    print LOG "find $inOptDir[$ind] -name autom4te.cache -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name autom4te.cache -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name config.log -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name config.log -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name config.status -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name config.status -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name Makefile -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name Makefile -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name '*~' -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name '*~' -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name '*%' -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name '*%' -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name '*#' -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name '*#' -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name \".deps\" -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name \".deps\" -exec rm -rf {} \\;");
    print LOG "find $inOptDir[$ind] -name \"SunWS_cache\" -exec rm -rf {} \\;\n";
    system("find $inOptDir[$ind] -name \"SunWS_cache\" -exec rm -rf {} \\;");
    if ($opt == 1) {
      # restore directory to original untouched state
      foreach $osL (@osList) {
        print LOG "find $inOptDir[$ind] -name \"*$osL\" -exec rm -rf {} \\;\n";
        system("find $inOptDir[$ind] -name \"*$osL\" -exec rm -rf {} \\;");
        print LOG "find $inOptDir[$ind] -name \"*$osL.a\" -exec rm -rf {} \\;\n";
        system("find $inOptDir[$ind] -name \"*$osL.a\" -exec rm -rf {} \\;");
        print LOG "find $inOptDir[$ind] -name \"*$osL.so\" -exec rm -rf {} \\;\n";
        system("find $inOptDir[$ind] -name \"*$osL.so\" -exec rm -rf {} \\;");
        $osLRev = reverse($osL);
        chop($osLRev);
        $osLNew = reverse($osLRev);
        print LOG "find .. -name \"$osLNew\" -exec rm -rf {} \\;\n";
        system("find .. -name \"$osLNew\" -exec rm -rf {} \\;");
      }
    }
  } elsif ($opt == 3) {
    # regular make install
    print LOG "make install\n";
    system("make install");
  } elsif ($opt == 6) {
    # regular make uninstall
    print LOG "make uninstall\n";
    system("make uninstall");
  } elsif ($opt == 4) {
    # regular make plus save libaries and executable compiled binaries with operating system name appended
    print LOG "make\n";
    system("make");
    @binDirs = `find $inOptDir[$ind] -name 'Makefile.am' -exec grep -l bin_ {} \\;`;
    $binCnt = 0;
    foreach $binDir (@binDirs) {
      $bDir[$binCnt++] = &getDir($binDir);
    }
    @bins = `find $inOptDir[$ind] -name 'Makefile.am' -exec grep bin_ {} \\; | grep -v \'\\\$\' | awk '{print \$3}'`;
    $binCnt = 0;
    foreach $bin (@bins) {
      chomp($bin);
      $bFile[$binCnt] = "$bDir[$binCnt]/$bin";
      $bFileNew[$binCnt] = sprintf("%s/%s-%s", $bDir[$binCnt], $bin, $OS);
      $binCnt++;
    }
    @libs = `find $inOptDir[$ind] -name '*.a' -print`;
    foreach $lib (@libs) {
      chomp($lib);

      # ignore $lib if it already has an operating system suffix
      $skip = 0;
      foreach $osL (@osList) {
	if ($lib =~ /$osL/) {
	  $skip = 1;
	}
      }
      next if ($skip);

      $bFile[$binCnt] = $lib;
      @arr = split(/\./, $lib);
      pop(@arr);
      $bFileNew[$binCnt] = join('.', @arr);
      $bFileNew[$binCnt] = sprintf("%s-%s.a", $bFileNew[$binCnt], $OS);
      $binCnt++;
    }
    @libs = `find $inOptDir[$ind] -name \"*.so\" -print`;
    foreach $lib (@libs) {
      chomp($lib);

      # ignore $lib if it already has an operating system suffix
      $skip = 0;
      foreach $osL (@osList) {
	if ($lib =~ /$osL/) {
	  $skip = 1;
	}
      }
      next if ($skip);

      $bFile[$binCnt] = $lib;
      @arr = split(/\./, $lib);
      pop(@arr);
      $bFileNew[$binCnt] = join('.', @arr);
      $bFileNew[$binCnt] = sprintf("%s-%s.so", $bFileNew[$binCnt], $OS);
      $binCnt++;
    }
    for ($jnd = 0; $jnd < $binCnt; $jnd++) {
      print LOG "mv $bFile[$jnd] $bFileNew[$jnd]\n";
      rename("$bFile[$jnd]" , "$bFileNew[$jnd]") || die "ERROR: Unable to rename $bFile[$jnd] to $bFileNew[$jnd]";
    }
  } else {
    # reconfigure
    print LOG "aclocal\n";
    system("aclocal");
    print LOG "autoconf\n";
    system("autoconf");
    print LOG "automake -a\n";
    system("automake -a");
    print LOG "./configure $CONF_DL\n";
    system("./configure $CONF_DL");
  }
  print LOG "cd $curDir\n";
  chdir("$curDir") || die "ERROR: Unable to chdir to $curDir";
}
print LOG "\n\n";
close(LOG);
exit 0;


############################### SUBROUTINES ###############################

sub formatInput
# this subroutine is used to format an input file or directory.  it changes
# <file> to <current directory>/<file> and ./<file> to
# <current directory>/<file>. it also recognizes ../, ../../, etc.
{
  my($val, $curDir) = @_;
  my(@arr, $revVal);
  while (($val =~ /^\.\.\//) || ($val =~ /^\.\.\$/)) {
    $revVal = reverse($val);
    chop($revVal);
    chop($revVal);
    chop($revVal);
    $val = reverse($revVal);
    $curDir = &getDir($curDir);
  }
  if (($val =~ /^.\//) || ($val =~ /^.$/)) {
    $revVal = reverse($val);
    chop($revVal);
    chop($revVal);
    $val = reverse($revVal);
  }
  if ($val !~ /^\//) {
    $val = sprintf("%s/%s", $curDir, $val);
  }
  while ($val =~ /\/$/) {
    chop($val);
      # remove tail / if one exists
  }
  while ($val =~ /\/\//) {
    $val =~ s|//|/|g;
  } return($val);
}

sub getDir
# this subroutine returns the directory of a full path file name
{
  my($val) = @_;
  my(@arr, $retVal);
  @arr = split('/', $val);
  pop(@arr);
  $retVal = join('/', @arr);
  return($retVal);
}
