#ifndef _DLContour_H_
#define _DLContour_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Contour Computation.
 *
 * The object is implemented as a static function, so no object of
 * class DLContour needs to be instantiated. See below for a usage example.
 */
class DLContour
{
   public:
   /** \cond **/
   /**
    * Default DLContour Constructor
    */
   DLContour();

   /**
    * Default DLContour Destructor
    */
   virtual ~DLContour();
   /** \endcond **/
	
   /**
    * Compute the contour image of a binary (black and white) image. 
    * A pixel will be set to black (foreground or "ink" color) if 
    * at least one of its neighbors (in the four principal directions) 
    * is part of the background (white).  The following code example will
    * return the contour image for <i>test.tif</i>:
    * \code
    * DLImage i ("test.tif");
    * DLImage contouredImage = DLContour::dlContourImage(i);
    * \endcode
    * \param  in const DLImage &
    * \return DLImage showing the contour of the image. 
    */
   static DLImage dlContourImage ( const DLImage & in );

   private:
   static DLImage dlContourGray   ( const DLImage & in );
   static DLImage dlContourBinary ( const DLImage & in );


};//end class

#endif //_DLContour_H_
