#ifndef _DLNORMALIZE_H_
#define _DLNORMALIZE_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Normalize an image.
 *
 * The algorithm is implemented as a static function, so no object of
 * class DLNormalize needs to be instantiated. See below for a usage example.
 */
class DLNormalize
{
   public:
   /** \cond **/
   /**
    * Default DLNormalize Constructor
    */
   DLNormalize();

   /**
    * Default DLNormalize Destructor
    */
   virtual ~DLNormalize();
   /** \endcond **/

   /**
    * Normalize the pixels of an image based on maximum and minimum
    * of its pixel values.
    * In a grayscale image, the minimum pixel value will be set to 0
    * and the maximum pixel value will be reset to 255 and the
    * intermediate values will be scaled accordingly.
    * For a color image, this process will be applied to each of the 
    * red, green and blue channels, separately.	The following code example will
    * return the normalized image for <i>test.tif</i>:
    * \code
    * DLImage i ("test.tif");
    * DLImage normalizedImage = DLNormalize::dlNormalizeImage(i);
    * \endcode
    * \param  in const DLImage &
    * \return DLImage of normalized image
    */
   static DLImage dlNormalizeImage ( const DLImage & in );

   private:

   static DLImage dlNormalizeGlobalImage ( const DLImage & );

};//end class

#endif //_DLNORMALIZE_H_
