#include "DLCommandLineParser.h"
#include "DLConnectedComponents.h"
#include "DLRectShape.h"
#include "DLLineShape.h"
#include "DLBitsPerPixelConverter.h"
#include "DLTextImage.h"
#include "DLSkeleton.h"
#include "DLSkew.h"
#include "DLPolar.h"
#include "DLNormalize.h"
#include "DLCartesian.h"
#include "DLContour.h"
#include "DLLogger.h"
#include "DLDilate.h"
#include "DLBlur.h"
#include "DLSharpen.h"
#include "DLErode.h"
#include "DLProfile.h"
#include "DLVersions.h"

#include <sys/stat.h>

#define FAILED_TO_OPEN -1


DLCommandLineParser::DLCommandLineParser() {
   loadImageAsMemory = false;
   //set default color to red
}
 
/**
 *The constructor for the CommandLineParser this handles all the parsing.
 *\param char *params[] The array holding the commandline arguments.
 *\param int paramCount The number of commandline arguments passed
 **/
//void DLCommandLineParser::processCommand(char * params[], int paramCount)
void DLCommandLineParser::processCommand(list <string> cmdList )
{

   // int successExcuted = false;
   while (cmdList.size() > 0)
   {
      try{
         if(dlOpenFile(cmdList))
            continue;
         if(dlLoadDocFile(cmdList))
            continue;
         if(dlCommandLineResize(cmdList))
            continue;
         if(dlCommandLinePolar(cmdList))
            continue;
         if(dlCommandLineCartesian(cmdList))
            continue;
         if(dlCommandLineOutput(cmdList))
            continue;
         if(dlCommandLineRotate(cmdList))
            continue;
         if(dlCommandLineSubImage(cmdList))
            continue;
         if(dlCommandLineFlip(cmdList))
            continue;
         if(dlCommandLineContour(cmdList))
            continue;
         if(dlCommandLineReverse(cmdList))
            continue;
         if(dlCommandLinePaste(cmdList))
            continue;
         if(dlCommandLineProject(cmdList))
            continue;
         if(dlCommandLineConvert(cmdList))
            continue;
         if(dlCommandLineSetPixel(cmdList))
            continue;
         if(dlCommandLineBoundCC(cmdList))
            continue;
         if(dlCommandLineDrawLine(cmdList))
            continue;
         if(dlCommandLineDrawBox(cmdList))
            continue;
         if(dlCommandLineDilate(cmdList))
            continue;
         if(dlCommandLineErode(cmdList))
            continue;
         if(dlCommandLineSharpen(cmdList))
            continue;
         if(dlCommandLineBlur(cmdList))
            continue;
         //if(dlCommandLineGaussianBlur(cmdList))
         //    continue;
         if(dlCommandLineToYCrCb(cmdList))
            continue;
         if(dlCommandLineYCrCbBinarization(cmdList))
            continue;
         if(dlDownscale256Color_global(cmdList))
            continue;
         if(dlCommandLinePercentThresholdBinarization(cmdList))
            continue;
         if(dlCommandLineThresholdBinarization(cmdList))
            continue;
         if(dlCommandLineColor2Gray_global(cmdList))
            continue;
         if(dlCommandLineGray2Binary_global(cmdList))
            continue;
         if(dlCommandLineBinary2Color(cmdList))
            continue;
         if(dlCommandLineGray2Color(cmdList))
            continue;
         if(dlCommandLineBinary2Gray(cmdList))
            continue;
         if(dlCommandLineSkeletonize(cmdList))
            continue;
         if(dlCommandLineDeskew(cmdList))
            continue;
         if(dlCommandLineNormalize(cmdList))
            continue;
         if(dlCommandLineFilterCC(cmdList))
            continue;
         if(dlCommandLineNiBlackBinarization(cmdList))
            continue;
         if(dlCommandLineFilterCCDensityLess(cmdList))
            continue;
         if(dlCommandLineFilterCCDensityGreater(cmdList))
            continue;
         if(dlCommandLineCentroid(cmdList))
            continue;
         if(dlCommandLineDebugSwitch(cmdList))
            continue;
         if(dlCommandLineAddImage(cmdList))
            continue;
         if(dlImageInfo(cmdList))
            continue;
         if(dlCommandLineVersionInfo(cmdList))
            continue;

         cout << "---->Invalid Command " << (string)*cmdList.begin() << endl;
         cmdList.pop_front();
      }
      catch (DLException E)
      {
         cout << E.dlGetErrorMsg() << endl;
      }
   }
}

/**
 *Handles the -output commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineOutput(list <string> &cmds) {

   bool excuted = false;

   string cmd = (string)*cmds.begin();
   if(cmd == "-output" )
   {
      excuted = true;
      cmds.pop_front();
      string fileName;
      try {
         //!		cout << "In output " << params[1] << endl;
         fileName = ((string)*cmds.begin()).c_str();
         cmds.pop_front();

         cout << "Saving Image : " << fileName << endl;
         image.dlSaveImage(fileName.c_str());
         //!		cout << "End output " << params[1] << endl;
      } catch (DLException E) {
         cout << "ERROR outputting to file " << fileName  << " " << E.dlGetErrorMsg() << endl;

      }
   }
   return excuted;
}

/**
 *Handles the -rotate commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineRotate(list <string> &cmds) {
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-rotate" )
   {
      excuted = true;
      cmds.pop_front();
      double angle = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Rotate image .. " << angle << " Degrees\n";
      image = image.dlRotateImage(angle);
   }


   //The angle of rotation read from the parameters.


   return excuted;
}

/**
 *Handles the -subimage commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineSubImage(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-subimage" )
   {
      excuted = true;
      int xcoor, ycoor;
      int width, height;

      cmds.pop_front();

      xcoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      ycoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      width = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      height = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Sub image ..\n";
      image = image.dlSubImage(xcoor, ycoor, width, height);
   }
   return excuted;
}

/**
 *Handles the -flip commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineFlip(list <string> &cmds) {

   bool executed = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-flip" )
   {
      executed = true;
      cmds.pop_front();
      string direction = (string)*cmds.begin();
      cmds.pop_front();

	  DLImage::DLFlipDirection enumDirection;
      if(direction == "DL_VERTICAL")
         enumDirection = DLImage::DL_VERTICAL;
      else if(direction == "DL_HORIZONTAL")
         enumDirection = DLImage::DL_HORIZONTAL;
	  else {
		cout << "Invalid Direction " << direction << endl;
		return executed;
	  }

      cout << "Flip image ..\n";
      image = image.dlFlipImage(enumDirection);
   }
   return executed;
}

/**
 *Handles the -contour commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read which is zero in this case.
 **/
bool DLCommandLineParser::dlCommandLineContour(list <string> &cmds) {
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-contour" )
   {
      excuted = true;
      cmds.pop_front();

      cout << "Contour image ..\n";
      //image = image.dlContourImage();
      image = DLContour::dlContourImage(image);
   }

   return excuted;
}

/**
 *Handles the -output commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLinePaste(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-paste" )
   {
      excuted = true;
      cmds.pop_front();
      string cutImageName = (string)(*cmds.begin()).c_str();
      cmds.pop_front();
      DLImage cutImage(cutImageName.c_str());
      int xcoor, ycoor;
      xcoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      ycoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Pasting image ..\n";
      image = image.dlPasteImage(cutImage, xcoor, ycoor);
   }

   return excuted;
}

/**
 *Handles the -project commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineProject(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-project" )
   {
      excuted = true;
      cmds.pop_front();
      string direction = (string)*cmds.begin();
	  DLProfile::DLProjectionDirection enumDirection;
      if(direction == "DL_VERTICAL")
         enumDirection = DLProfile::DL_VERTICAL;
      if(direction == "DL_HORIZONTAL")
         enumDirection = DLProfile::DL_HORIZONTAL;

      cmds.pop_front();
      cout << "Profile image ..\n";
		
      vector<int> prof;

      switch ( enumDirection )
      {
	  case DLProfile::DL_VERTICAL:
            // vector<int> prof = image.dlProjectProfile(intDirection);
            prof = DLProfile::dlProjectVerticalProfile(image);
            break;
		 case DLProfile::DL_HORIZONTAL:
            prof = DLProfile::dlProjectHorizontalProfile(image);
            break;
         default:
			 cout << "Invalid Profile direction: " << direction << endl;
      }
   }

   return excuted;
}

/**
 *Handles the -convert commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineConvert(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-convert" )
   {
      excuted = true;
      cmds.pop_front();
      string conversionCode = (string)*cmds.begin();
      cmds.pop_front();
	  DLBitsPerPixelConverter::DLImageConversions code;
      if(conversionCode == "DL_2BINARY")
         code = DLBitsPerPixelConverter::DL_2BINARY;
      else if(conversionCode == "DL_2GRAY")
         code = DLBitsPerPixelConverter::DL_2GRAY;
      else if(conversionCode == "DL_2COLOR")
         code = DLBitsPerPixelConverter::DL_2COLOR;
	  else {
		  cout << "Invalid Conversion Code: " << conversionCode << endl;
		  return excuted;
	  }
      cout << "Converting image ..\n";
      image = DLBitsPerPixelConverter::dlConvertImage(image, code);
   }
   return excuted;
}

/**
 *Handles the -setpixel commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineSetPixel(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-setPixel" )
   {
      excuted = true;
      cmds.pop_front();
      int x, y;
      int r, g, b;
      //the coordinates to set pixel color to.
      x = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      y = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      //the color to set those coordinates.
      r = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      g = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      b = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      //image = image.dlSetPixel(x, y, r, g, b);

      cout << "Setting pixel... " << x << " " << y << " " << r << " " <<  g << " " << b << endl;
      image.dlSetPixel(x, y, r, g, b);

   }
   return excuted;
}

/**
 *Handles the -boundCC commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineBoundCC(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-boundCC" )
   {

      excuted = true;
      cmds.pop_front();
      cout << "Bounding CC.." << endl;
      DLImage colorImg = DLBitsPerPixelConverter::dlConvertImage(image, DLBitsPerPixelConverter::DL_2COLOR);
      DLConnectedComponents* cc = new DLConnectedComponents();
      cc->dlCreateConnectedComponents(image);
      vector <DLComponent> ccList = cc->dlGetConnectedComponents(); 
      vector<DLComponent>::iterator iterf = ccList.begin();

      DLColor lineColor(255,0,0);
      for(; iterf != ccList.end(); iterf++) {	
         DLComponent cp = (DLComponent)*iterf;
         DLRectShape rect(cp.dlGetColMin(), cp.dlGetRowMin(), 
                          cp.dlGetColMax() - cp.dlGetColMin() + 1, 
                          cp.dlGetRowMax() - cp.dlGetRowMin() + 1);
         //rect.dlSetFillOff();
			
         rect.dlSetLineColor(lineColor);
         rect.dlDrawShape(&colorImg);
      }
      image = colorImg;
      //cout << "End boundCC" << endl;
      delete cc;
   }
   return excuted;
}


bool DLCommandLineParser::dlCommandLineFilterCC(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-filterCCLessThan" )
   {

      excuted = true;
      cmds.pop_front();
      float filterSize = (float)atof(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      cout << "Filtering CC.." << endl;
      DLConnectedComponents* cc = new DLConnectedComponents();
      cc->dlCreateConnectedComponents(image);
      cc->dlFilterCCMassLess(filterSize);
       
      image = cc->dlRenderFilteredCCs();
      //cout << "End boundCC" << endl;
      delete cc;
   }
   return excuted;
}


bool DLCommandLineParser::dlCommandLineFilterCCDensityGreater(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-filterCCDensityGreaterThan" )
   {

      excuted = true;
      cmds.pop_front();
      float filterSize = (float)atof(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      cout << "Filtering CC.." << endl;
      DLConnectedComponents* cc = new DLConnectedComponents();
      cc->dlCreateConnectedComponents(image);
      cc->dlFilterCCDensityGreater(filterSize);
      
      image = cc->dlRenderFilteredCCs();
      //cout << "End boundCC" << endl;
      delete cc;
   }
   return excuted;
}

bool DLCommandLineParser::dlCommandLineFilterCCDensityLess(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-filterCCDensityLessThan" )
   {

      excuted = true;
      cmds.pop_front();
      float filterSize = (float)atof(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      cout << "Filtering CC.." << endl;
      DLConnectedComponents* cc = new DLConnectedComponents();
      cc->dlCreateConnectedComponents(image);
      cc->dlFilterCCDensityLess(filterSize);
      
      image = cc->dlRenderFilteredCCs();
      //cout << "End boundCC" << endl;
      delete cc;
   }
   return excuted;
}
/**
 *Handles the -drawline commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineDrawLine(list <string> &cmds) 
{
   //coordinates of the line to draw.
   // int num_colors_defined = 3; //red, green,blue
   bool excuted = false;
   //int colorIndex=0;
   int p1=0, p2, p3, p4, p5;
   DLColor lineColor;
   bool isColor = true;
   string cmd = (string)*cmds.begin();
   if(cmd == "-drawLine" )
   {
      excuted = true;
      cmds.pop_front();
      string color = ((string)*cmds.begin()).c_str();
      if (!color.compare("DL_RED"))
         lineColor.dlSetColor(DLColor::DL_RED1);
//			p1 = DL_RED;
      else if (!color.compare("DL_GREEN"))
         lineColor.dlSetColor(DLColor::DL_GREEN1);
//			p1 = DL_GREEN;
      else if (!color.compare("DL_BLUE"))
         //p1 = DL_BLUE;
         lineColor.dlSetColor(DLColor::DL_COLOR_BLUE);
      else
      {
         p1  = atoi(((string)*cmds.begin()).c_str());
         isColor = false;
      }
      cmds.pop_front();
      p2  = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      p3 = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      p4 = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      DLImage colorImage = DLBitsPerPixelConverter::dlConvertImage(image, DLBitsPerPixelConverter::DL_2COLOR);
      DLRectShape box;
      DLLineShape line;
	
      if (isColor) // not red, green, blue. Must be a coordinate
      {
         p5 = atoi(((string)*cmds.begin()).c_str());
         DLLineShape lineTemp(p2, p3, p4, p5);

         //if (p1 == DL_GREEN)
         //             lineTemp.dlSetLineColor(DLColor(DL_DARKGREEN));
         //else if (p1 == DL_RED)
         //	lineTemp.dlSetLineColor(DLColor(DL_RED1));
         //else if (p1 == DL_BLUE)
         //	lineTemp.dlSetLineColor(DLColor(DL_ALICEBLUE));
         lineTemp.dlSetLineColor(lineColor);

         line = lineTemp;
         cmds.pop_front();
      }
      else
      {
         DLLineShape lineTemp(p1, p2, p3, p4);
         line = lineTemp;
      }

      //cout << "Draw Line on image Image.. X1 =" << p1 << " y1=" << p2 << " x2=" << x2 << " y2=" << y2<< endl;
      line.dlDrawShape(&colorImage);
      image = colorImage;
   }


   return excuted;
}


/**
 *Handles the -drawBox commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineDrawBox(list <string> &cmds) 
{
   //coordinates of the line to draw.
   // int num_colors_defined = 3; //red, green,blue
   bool excuted = false;
   bool isColor = true;
   //int colorIndex=0;
   int p1=0, p2, p3, p4, p5;
   DLColor lineColor;
   string cmd = (string)*cmds.begin();
   if(cmd == "-drawBox" )
   {
      excuted = true;
      cmds.pop_front();
      string color = ((string)*cmds.begin()).c_str();
      if (!color.compare("DL_RED"))
         lineColor.dlSetColor(DL_RED1);
//			p1 = DL_RED;
      else if (!color.compare("DL_GREEN"))
         lineColor.dlSetColor(DL_GREEN1);
//			p1 = DL_GREEN;
      else if (!color.compare("DL_BLUE"))
         //p1 = DL_BLUE;
         lineColor.dlSetColor(DL_COLOR_BLUE);
      else
      {
         p1  = atoi(((string)*cmds.begin()).c_str());
         isColor = false;
      }
      cmds.pop_front();
      p2  = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      p3 = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      p4 = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      DLImage colorImage = DLBitsPerPixelConverter::dlConvertImage(image, DLBitsPerPixelConverter::DL_2COLOR);
      DLRectShape box;
	
      if (isColor) // not red, green, blue. Must be a coordinate
      {
         p5 = atoi(((string)*cmds.begin()).c_str());
         DLRectShape boxTemp(p2, p3, p4, p5);
         //if (p1 == DL_GREEN)
         //	boxTemp.dlSetLineColor(DLColor(DL_DARKGREEN));
         //else if (p1 == DL_RED)
         //	boxTemp.dlSetLineColor(DLColor(DL_RED1));
         //else if (p1 == DL_BLUE)
         //	boxTemp.dlSetLineColor(DLColor(DL_ALICEBLUE));
         boxTemp.dlSetLineColor(lineColor);
         box = boxTemp;
         cmds.pop_front();
      }
      else
      {
         DLRectShape boxTemp(p1, p2, p3, p4);
         box = boxTemp;
      }

      //cout << "Draw Line on image Image.. X1 =" << p1 << " y1=" << p2 << " x2=" << x2 << " y2=" << y2<< endl;
      box.dlDrawShape(&colorImage);
      image = colorImage;
   }


   return excuted;
}

/**
 *Handles the -reverse commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineReverse(list <string> &cmds) {


   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-reverse" )
   {
      excuted = true;
      cmds.pop_front();

      cout << "Reverse Image.." << endl;
      image = image.dlReverseImage();
   }
   return excuted;
}

/**
 *Handles the -resize commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineResize(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-resize" )
   {
      excuted = true;
      cmds.pop_front();
      int firstParam = atoi(((string)*cmds.begin()).c_str());

      if ( firstParam == 0 ) {
         double factor = atof(((string)*cmds.begin()).c_str());
         cmds.pop_front();

         cout << "Resize Image.. factor=" << factor << endl;
         image = image.dlResizeImage(factor);
      }
      else {
         int w, h;
         w = firstParam;
         cmds.pop_front();
         h = atoi(((string)*cmds.begin()).c_str());
         cmds.pop_front();

         cout << "Resize Image.. w=" << w << " h= " << h << endl;
         image = image.dlResizeImage(w, h);
      }
   }

   //means an error occured reading the second param to resize
   //so assumes dlResizeImage(double factor) is supposed to be
   //called

   return excuted;
}

/**
 *Handles the -normalize commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineNormalize(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-normalize" )
   {
      excuted = true;
      cmds.pop_front();
	
      cout << "Normalizing Image ..... " << endl;
      image = DLNormalize::dlNormalizeImage(image);
   }

   //means an error occured reading the second param to resize
   //so assumes dlResizeImage(double factor) is supposed to be
   //called

   return excuted;
}

/**
 *Handles the -normalize commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineCartesian(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-cartesian" )
   {
      excuted = true;
      cmds.pop_front();
	
      cout << "Cartesian Image Creation ..... " << endl;
      image = DLCartesian::dlCartesianImage(image);
      //image = image.dlCartesianImage();
   }

   //means an error occured reading the second param to resize
   //so assumes dlResizeImage(double factor) is supposed to be
   //called

   return excuted;
}

/**
 *Handles the -polar commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLinePolar(list <string> &cmds) {
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-polar" )
   {
      excuted = true;
      cmds.pop_front();

      if ( cmds.size() > 0 )
      {
         string flag = ((string)*cmds.begin()).c_str();
	
         if ( flag.compare( "true" ) == 0 || flag.compare( "TRUE" ) == 0 
              || flag.compare( "T" ) == 0 || flag.compare( "t" ) == 0
              || flag.compare( "1" ) == 0 )
         {
            cout << "Polarizing Image with flag = true ..... " << endl;
            cmds.pop_front();
            image = DLPolar::dlPolarImage(image, true);
         }
         else if ( flag.compare( "false" ) == 0 || flag.compare( "FALSE" ) == 0 
                   || flag.compare( "F" ) == 0 || flag.compare( "f" ) == 0
                   || flag.compare( "0" ) == 0 )
         {
            cout << "Polarizing Image with flag = false ..... " << endl;
            cmds.pop_front();
            image = DLPolar::dlPolarImage(image, true);
         }
         else
         {
            cout << "flag not understood.  Acceptable terms:\n";
            cout << "\tTRUE | FALSE" << endl;
            cmds.pop_front();
         }
      }
      else
      {
         cout << "Polarizing Image with flag = true ..... " << endl;
         image = DLPolar::dlPolarImage(image, true);
      }
   }

   //means an error occured reading the second param to resize
   //so assumes dlResizeImage(double factor) is supposed to be
   //called

   return excuted;
}

bool DLCommandLineParser::dlCommandLineDilate(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dilate" )
   {
      excuted = true;
      cmds.pop_front();
      int times = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Dilate.. " << times << " times" << endl;
      image = DLDilate::dlDilate(this->image, times);
   }
   return excuted;

}
bool DLCommandLineParser::dlCommandLineErode(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-erode" )
   {
      excuted = true;
      cmds.pop_front();
      int times = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Erode.. " << times << " times" << endl;
      image = DLErode::dlErode(this->image, times);
   }
   return excuted;
}
bool DLCommandLineParser::dlCommandLineSharpen(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-sharpen" )
   {
      excuted = true;
      cmds.pop_front();
      int times = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Sharpen.. " << times << " times" << endl;
      image = DLSharpen::dlSharpen(this->image, times);
   }

   return excuted;
}
bool DLCommandLineParser::dlCommandLineBlur(list <string> &cmds)
{

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-blur" )
   {
      excuted = true;
      cmds.pop_front();
      int times = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Blur.. " << times << " times" << endl;
      image = DLBlur::dlBlur(this->image, times);
   }
   return excuted;
}


bool DLCommandLineParser::dlCommandLineToYCrCb(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-toYCrCb" )
   {
      excuted = true;
      cmds.pop_front();
      cout << "Converting YCrCb.. " << endl;
      image = this->image.dlConvertRGBToYCrCb();
   }
   return excuted;
}


bool DLCommandLineParser::dlCommandLineYCrCbBinarization(list <string> &cmds)
{
   bool excuted = false;

   string cmd = (string)*cmds.begin();
   if(cmd == "-dlYCrCbBinarization" )
   {
      excuted = true;
      cmds.pop_front();
      cout << "YCrCbBinarization.. " << endl;

      image = DLBitsPerPixelConverter::dlDownscaleYCrCbBinarization(image);
   }

   return excuted;
}


bool DLCommandLineParser::dlCommandLineNiBlackBinarization(list <string> &cmds)
{
   bool excuted = false;

   string cmd = (string)*cmds.begin();
   if(cmd == "-dlNIBlackBinarization" )
   {
      excuted = true;
      cmds.pop_front();
      cout << "NiBlack Binarization.. " << endl;

      image = DLBitsPerPixelConverter::dlDownscaleGray2Binary_niblack(image);
   }

   return excuted;
}

bool DLCommandLineParser::dlDownscale256Color_global(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dl256Color" )
   {
      excuted = true;
      cmds.pop_front();
      int quant_tmp = atoi(((string)*cmds.begin()).c_str());
      bool quant;
      if(quant_tmp == 1)
         quant = true;
      else
         quant = false;

      cmds.pop_front();

      cout << "Downscale256Color " << quant << endl;
      image = DLBitsPerPixelConverter::dlDownscale256Color_global(image, quant);
   }

   return excuted;

}
bool DLCommandLineParser::dlCommandLinePercentThresholdBinarization(list <string> &cmds)
{

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlPercentThresholdBinarization" )
   {
      excuted = true;
      cmds.pop_front();
      double percent = atof(((string)*cmds.begin()).c_str());
      int intPercent = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      if ( intPercent > 0 ) {

         cout << "Percent Threshold Binarization " << intPercent << endl;

         image = DLBitsPerPixelConverter::dlDownscalePercentThresholdBinarization(image, intPercent);
      }
      else
      {
         cout << "Percent Threshold Binarization " << percent << endl;

         image = DLBitsPerPixelConverter::dlDownscaleColor2Binary_percentThreshold(image, percent);
      }	
   }

   return excuted;
}
bool DLCommandLineParser::dlCommandLineThresholdBinarization(list <string> &cmds)
{

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlThresholdBinarization" )
   {
      excuted = true;
		
      cmds.pop_front();
      int rthresh = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      int gthresh = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      int bthresh = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      cout << "Threshold Binarization " << rthresh << " " << gthresh << " " << bthresh << endl;
      image = DLBitsPerPixelConverter::dlDownscaleColor2Binary_threshold(image, rthresh,gthresh,bthresh);
   }

   return excuted;
}

bool DLCommandLineParser::dlCommandLineColor2Gray_global(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlColor2Gray_global" )
   {
      excuted = true;
      cout << "Color2Gray.." << endl;
      cmds.pop_front();
      image = DLBitsPerPixelConverter::dlDownscaleColor2Gray_global(image);
   }
   return excuted;
}
bool DLCommandLineParser::dlCommandLineGray2Binary_global(list <string> &cmds)
{

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlGray2Binary_global" )
   {
      excuted = true;
      cout << "Gray2Binary.." << endl;
      cmds.pop_front();
      int thresh = atoi(((string)*cmds.begin()).c_str());
      image = DLBitsPerPixelConverter::dlDownscaleGray2Binary_global(image, thresh);
      cmds.pop_front();
   }

   return excuted;
}
bool DLCommandLineParser::dlCommandLineBinary2Color(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlBinary2Color" )
   {
      excuted = true;
      cout << "Binary2Color.." << endl;
      cmds.pop_front();
      image = DLBitsPerPixelConverter::dlBinary2Color(image);
   }


   return excuted;
}
bool DLCommandLineParser::dlCommandLineGray2Color(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlGray2Color" )
   {
      excuted = true;
      cout << "Gray2Color.." << endl;
      cmds.pop_front();
      image = DLBitsPerPixelConverter::dlGray2Color(image);
   }
   return excuted;
}
bool DLCommandLineParser::dlCommandLineBinary2Gray(list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-dlBinary2Gray" )
   {
      excuted = true;
      cout << "Binary2Gray.." << endl;
      cmds.pop_front();
      image = DLBitsPerPixelConverter::dlBinary2Gray(image);
   }
   return excuted;
}
/**
 *Opens a file for manipulation.
 *
 *\param inImage char* The name of the file to open.
 *
 **/
bool DLCommandLineParser::dlOpenFile(list <string>& cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-input")
   {
      excuted = true;
      cmds.pop_front();
		
      if ( cmds.size() < 1 ) 
      {
         throw DLException(DL_FILE_NOT_FOUND_EXCEPTION, "<DLCommandLineParser:dlOpenFile(list<string>&)> Could not open incoming image", __FILE__, __LINE__);
      }
      string fname = (string)*cmds.begin();
      cout << "Opening " << fname << endl;
      cmds.pop_front();
      // force the load to load a memory block
      if(this->loadImageAsMemory)
      {
         struct stat results;

         if (stat(fname.c_str(), &results) == 0)
         {
            // The size of the file in bytes is in
            // results.st_size
         }
         else
         {
            throw DLException(DL_FILE_NOT_FOUND_EXCEPTION, "<DLCommandLineParser:dlOpenFile(list<string>&)> Could not open incoming image", __FILE__, __LINE__);
         }
         // Get the size of the input file
         int size = results.st_size;  
         cout << "file size = " << size << endl;


         // Open file in bin mode
         ifstream infile ( fname.c_str(),  ios::in | ios::binary);

         if(infile)
         {
            char* buff = new char[size];
            try{

               // Read in the file into a buffer
               if (!infile.read (buff, size)) 
               {
                  cout << "Can't read file " << fname << endl;
               }

               // Load the image using the char* buffer
               image =  DLImage (buff,size);
   
            }catch(...)
            {
               if(buff!= NULL)
                  delete[] buff;
               throw;
            }

            if(buff!= NULL)
               delete[] buff;
         }

      }
      // load file using DLIMage
      else
      {
         image = DLImage(fname.c_str());
      }
      dlLogger.dlLog(DL_INFO, "Dimensions: %dx%d\n", 
                     image.dlGetWidth(),
                     image.dlGetHeight());
      dlLogger.dlLog(DL_INFO, "Resolution: %dx%d\n", 
                     image.dlGetHorizontalResolution(),
                     image.dlGetVerticalResolution());
      dlLogger.dlLog(DL_INFO, "BitDepth: %d\n", 
                     image.dlGetDepth());
   }

   return excuted;
}

/**
 *Opens a document image file for manipulation.
 *
 *\param inImage char* The name of the file to open.
 *
 **/
bool DLCommandLineParser::dlLoadDocFile(list <string>& cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-loadDocFile")
   {
      excuted = true;
      cmds.pop_front();

      if ( cmds.size() < 1 ) 
      {
         throw DLException(DL_FILE_NOT_FOUND_EXCEPTION, "<DLCommandLineParser:dlOpenFile(list<string>&)> Could not open incoming docFile", __FILE__, __LINE__);
      }
      else
      {
         string fname = (string)*cmds.begin();
         cmds.pop_front();
         cout << "Opening " << fname << endl;

         image = DLTextImage(fname.c_str());
      }
   }

   //!	cout << "In openfile " << inImage << endl;
   //image = DLImage(cmds.);
   //!	cout << "end openfile " << inImage << endl;
   return excuted;
}

bool DLCommandLineParser::dlCommandLineSkeletonize(list <string>& cmds) {
   bool executed = false;
   string cmd = (string)*cmds.begin();
   if ( cmd == "-dlSkeletonize" ) {
      executed = true;
      cmds.pop_front();
      //DLSkeleton dlc;
      //DLImage skeleton = dlc.dlExtractSkeleton(image);
      DLImage skeleton = DLSkeleton::dlExtractSkeletonRutovitz(image);
      image = skeleton.dlCopyImage();
   }
   return executed;
}

/**
 *Handles the -deskew commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineDeskew(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-deskew" )
   {
      excuted = true;
      cmds.pop_front();

      cout << "Deskew Image.." << endl;
      // double lowerBound = -90;
      // double upperBound = 82;
      // double delta = 1;
      // double skew = image.dlDetectSkew(lowerBound, upperBound, delta);
      double skew = DLSkew::dlDetectSkew(image);//, lowerBound, upperBound, delta);
      cout << "Skew angle " << skew << endl;
      image = image.dlRotateImage(-skew);
   }
   return excuted;
}

bool DLCommandLineParser::dlCommandLineCentroid ( list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-centroid" )
   {
      DLPoint centroid;

      /*unsigned long int x;
        cout << sizeof(x) << endl;*/

      excuted = true;
      cmds.pop_front();

      cout << "Calculating Centroid.." << endl;

      centroid = image.dlCentroid();
	
      cout << "\tCentroid: {" << centroid.x 
           << ", " << centroid.y << "}" << endl;
   }
   return excuted;
}

bool DLCommandLineParser::dlCommandLineDebugSwitch( list <string> &cmds)
{
   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-logger" )
   {
      excuted = true;
      cmds.pop_front();
      string dbgType = (string)*cmds.begin();
      cmds.pop_front();

      if(dbgType == "DEBUG")
      {
         dlLogger.dlSetLogSeverity(DL_DEBUG);
         cout << "Changed logger severity to DL_DEBUG\n";
      }
      else if (dbgType == "INFO")
      {
         dlLogger.dlSetLogSeverity(DL_INFO);
         cout << "Changed logger severity to DL_INFO\n";
      }

      else if(dbgType == "CRITICAL")
      {
         dlLogger.dlSetLogSeverity(DL_CRITICAL);
         cout << "Changed logger severity to DL_CRITICAL\n";
      }
      else if(dbgType == "OFF")
      {
         dlLogger.dlSetLogSeverity(DL_LOG_OFF);
         cout << "Changed logger severity to DL_LOG_OFF\n";
      }
      else
      {
         cout << "Unknown logger severity '" << dbgType
              << "' please enter DEBUG, INFO, or CRITICAL\n" ;
      }

   }
   else if(cmd == "-dbg_loadAsMemory" )
   {
      excuted = true;
      cmds.pop_front();
      string dbgType = (string)*cmds.begin();
      cmds.pop_front();

      if(dbgType == "YES")
      {
         this->loadImageAsMemory = true;
         cout << "Loading images from memory...\n";
      }
      else if(dbgType == "NO")
      {
         this->loadImageAsMemory = false;
         cout << "Loading images from file...\n";
      }
      else 
      {
         cout << "Invalid argument for command : -dbg_loadAsMemory\n";
         cout << "-dbg_loadAsMemory YES|NO\n";
      }
 
   }

   return excuted;
}

/**
 *Handles the -output commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineAddImage(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-addImage" )
   {
   
      excuted = true;
      cmds.pop_front();
      string imageName = (string)(*cmds.begin()).c_str();
      cmds.pop_front();
   
      DLImage cutImage(imageName.c_str());
      int xcoor, ycoor;
      xcoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();
      ycoor = atoi(((string)*cmds.begin()).c_str());
      cmds.pop_front();

      cout << "Adding image ..\n";
      image = image.dlAddImage(cutImage, xcoor, ycoor);
   
   }

   return excuted;
}
/**
 *Handles the -output commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlImageInfo(list <string> &cmds) {

   bool excuted = false;
   string cmd = (string)*cmds.begin();
   if(cmd == "-imageInfo" )
   {
   
      excuted = true;
      cmds.pop_front();

      DLImageHead head = image.dlGetImageHead();
       
      ///image width
      cout << "W=" <<head.width << endl;

      ///image height
      cout << "H="<<head.height << endl;

      ///image depth
      cout << "color depth="<<head.depth << endl;

      ///image channels
      cout << "Channels=" << head.channels  << endl;

      ///image colorModel
      cout <<"Color Model=" <<head.colorModel  << endl;

      ///image rowSize
      cout << "Row Size=" <<head.rowSize  << endl;

      ///image resolution
      cout << "Horizontal Res=" <<head.horizontalResolution  << endl;

      ///image resolution
      cout << "Vertical Res="<<head.verticalResolution << endl;

   }

   return excuted;
}

/**
 *Handles the -v, --version, -version commandline option
 *
 *\params char*[] The params passed from the commandline
 *\return the number of params this function read.
 **/
bool DLCommandLineParser::dlCommandLineVersionInfo(list <string> &cmds) {

   bool executed = false;
   string cmd = (string) *cmds.begin();

   if (cmd == "-version" || cmd == "--version" || cmd == "-v") {
      executed = true;
      cmds.pop_front();

      DLVersions::dlPrintAllVersionInfo();
   }

   return executed;
}

/************************************************************************ *
FILE: $RCSfile: DLCommandLineParser.cpp,v $ 
DESCRIPTION: 

* ************************************************************************ *
Modification Log: $Log: DLCommandLineParser.cpp,v $
Modification Log: Revision 1.36  2007/03/20 18:33:44  meyers_tim
Modification Log: Fixed DLColor qualifier issue
Modification Log:
Modification Log: Revision 1.35  2007/03/19 19:23:33  meyers_tim
Modification Log: Removed deprecated functions that were moving into step 3 of the documented deprecation process outlined during the February 2006 meeting with UMD.
Modification Log:
Modification Log: Revision 1.34  2007/03/14 20:22:57  meyers_tim
Modification Log: updated DLDirection enum with more specific enums
Modification Log:
Modification Log: Revision 1.33  2007/03/13 20:02:51  meyers_tim
Modification Log: Added Qualifiers re bug #691
Modification Log:
Modification Log: Revision 1.32  2007/02/28 21:11:18  meyers_tim
Modification Log: autoformatted
Modification Log:
Modification Log: Revision 1.31  2007/02/26 17:25:32  jjbran2
Modification Log: corrected colorspace conversion function names per bug #531
Modification Log: and started deprecation process for original functions
Modification Log:
Modification Log: Revision 1.30  2007/02/14 22:17:31  meyers_tim
Modification Log: fixed issue with DLVersion object
Modification Log:
Modification Log: Revision 1.29  2006/12/27 19:26:13  meyers_tim
Modification Log: exporting updates from client site
Modification Log:
Modification Log: Revision 1.28  2006/08/29 19:46:22  meyers_tim
Modification Log: ran through emacs autoindent
Modification Log:
Modification Log: Revision 1.27  2006/05/23 20:33:06  meyers_tim
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.26  2006/05/17 19:16:57  meyers_tim
Modification Log: Added ability to turn off Logger Warnings
Modification Log:
Modification Log: Revision 1.25  2006/05/15 19:16:45  meyers_tim
Modification Log: Updated some method calls to reflect recent API changes
Modification Log:
Modification Log: Revision 1.24  2006/05/11 18:27:44  meyers_tim
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.23  2006/05/11 15:32:28  meyers_tim
Modification Log: fixed boundCC issue
Modification Log:
Modification Log: Revision 1.22  2006/03/13 20:00:00  meyers_tim
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.21  2006/03/08 21:17:25  meyers_tim
Modification Log: Adding new features
Modification Log:
Modification Log: Revision 1.20  2006/03/07 21:54:32  meyers_tim
Modification Log: changed deprecated functions to use new ones
Modification Log: DLPolar and DLNormalize
Modification Log:
Modification Log: Revision 1.19  2006/03/07 21:39:05  meyers_tim
Modification Log: Added DLNormalize and DLPolar objects
Modification Log:
Modification Log: Added their functionality to the regression tests.
Modification Log:
Modification Log: Updated GroundTruth Data
Modification Log:
Modification Log: Revision 1.18  2006/02/16 16:32:00  chen_kevin
Modification Log: removed color bounding boxes for dlCommandLineFilterCC function
Modification Log:
Modification Log: Revision 1.17  2006/02/10 18:11:36  chen_kevin
Modification Log: Added "-imageInfo" to print the current image header info
Modification Log:
Modification Log: Revision 1.16  2005/11/10 15:15:00  chen_kevin
Modification Log: removed cout.flush
Modification Log:
Modification Log: Revision 1.15  2005/11/10 15:06:10  chen_kevin
Modification Log: removed couts
Modification Log:
Modification Log: Revision 1.14  2005/11/10 15:04:04  chen_kevin
Modification Log: Added addImage command to commandline
Modification Log:
Modification Log: Revision 1.13  2005/10/03 17:33:07  meyers_tim
Modification Log: fixed dlOpenImage so that it throws an exception when
Modification Log: a file cannot be found when loading from memory.
Modification Log: Previously it simply cout-ed an error message
Modification Log:
Modification Log: Revision 1.12  2005/09/21 22:47:31  meyers_tim
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.11  2005/09/13 23:30:53  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.10  2005/09/08 21:28:35  chen_kevin
Modification Log: modified all functions to dlXXXXX where XXXX is the function name.  Also, changed function names to be consistent with the naming of all DOCLIB functions.  Bug : 504
Modification Log:
Modification Log: Revision 1.9  2005/09/06 19:57:39  chen_kevin
Modification Log: changed DlGetErrMsg to dlGetErrorMsg
Modification Log:
Modification Log: Revision 1.8  2005/06/29 21:52:23  chen_kevin
Modification Log: Added the ability to force the commandline object to use memory based loading.
Modification Log:
Modification Log: Revision 1.7  2005/06/27 19:30:46  chen_kevin
Modification Log: Added command to turn on debugger on command line.
Modification Log:
Modification Log: Revision 1.6  2005/06/15 22:03:28  chen_kevin
Modification Log: Added a color when drawing a line
Modification Log:
Modification Log: Revision 1.5  2005/05/19 21:25:33  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.4  2005/04/27 19:08:51  chen_kevin
Modification Log: Removed DLDegradation
Modification Log:
Modification Log: Revision 1.3  2005/04/20 19:45:31  chen_kevin
Modification Log: Added valid angles (-45 to 45) for deskew.  Also, overloaded the DLDeskew to have a angle range of -20 to 20 and inc of 1 degree
Modification Log:
Modification Log: Revision 1.2  2005/04/12 15:52:17  chen_kevin
Modification Log: Modified to point to new regression test script
Modification Log:
Modification Log: Revision 1.1  2005/04/11 17:54:39  chen_kevin
Modification Log: This file was moved from ..\doclib\DoclibTools\DoclibCommandLine directory back in DOCLIB_before_reorg_4_11 tag.  To access the old change logs, please checkout tag version of DOCLIB "DOCLIB_before_reorg_4_11 tag"
Modification Log:
Modification Log: Revision 1.22  2005/04/01 19:32:06  meyers_tim
Modification Log: Added centroid calculation.  command is:
Modification Log: -centroid
Modification Log:
Modification Log: Revision 1.21  2005/03/08 20:16:55  meyers_tim
Modification Log: In reference to bug 0000359
Modification Log: dlOpenFile(list) attempted to access front of empty list and would core dump.
Modification Log: dlLoadDocFile(list) suffered same problem and was also fixed.
Modification Log:
Modification Log: Revision 1.20  2005/02/08 14:55:11  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.19  2005/02/02 16:58:41  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.18  2005/01/31 20:47:18  reddyc
Modification Log: DLCommandLineParser.cpp had a bug in the skeletonization call.. FIXED
Modification Log:
Modification Log: Revision 1.17  2005/01/31 20:35:43  reddyc
Modification Log: Updated the name of the skeletonization functions and added code to convert the binary image back to its binary image format.  The skeletonization algorithm used to convert it to a grayscale and return the grayscale image back instead of converting it back to a binary image.
Modification Log:
Modification Log: Revision 1.16  2005/01/31 19:59:37  reddyc
Modification Log: Removed extra skeleton classes...made the dlExtractSkeleton method static..
Modification Log:
Modification Log: Revision 1.15  2005/01/24 16:15:25  sdsampa
Modification Log: consolidated skew functions into its own skew class. Meanwhile, existing functions within DLImage has been deprecated (exception is thrown asking user to reference DLSkew class.
Modification Log:
Modification Log: Revision 1.14  2005/01/21 16:02:18  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.13  2005/01/14 18:41:58  chen_kevin
Modification Log: Added new command to doclib commandline.
Modification Log:
Modification Log: Revision 1.12  2004/12/01 20:49:32  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.11  2004/11/11 19:06:46  chen_kevin
Modification Log: DLDocumentImage to DLTextImage
Modification Log:
Modification Log: Revision 1.10  2004/11/02 14:37:43  chen_kevin
Modification Log: Added color support for -drawLine
Modification Log:
Modification Log: Revision 1.9  2004/10/28 04:07:08  sdsampa
Modification Log: support dlDrawBox
Modification Log:
Modification Log: Revision 1.8  2004/10/26 02:35:58  sdsampa
Modification Log: support dlDrawBox
Modification Log:
Modification Log: Revision 1.7  2004/09/20 03:06:47  zhugy
Modification Log: Restore -deskew option to the commandline for image deskewing
Modification Log:
Modification Log: Revision 1.6  2004/07/15 14:44:12  sdsampa
Modification Log: added documentation
Modification Log:
Modification Log: Revision 1.5  2004/07/12 23:15:32  reddyc
Modification Log: Implemention of the skeletonization algorithim from:
Modification Log: http://www.csc.liv.ac.uk/~prima/ICDAR2003/Papers/0143_512_huang_l.pdf
Modification Log:
Modification Log: Added some changes to the DLCommandLine in the protoType folder, not done yet.
Modification Log:
Modification Log: Revision 1.4  2004/07/06 14:53:24  sdsampa
Modification Log: update cvs log
Modification Log: 

* ************************************************************************ *
DocLib is being developed under contract by a collaboration between:

The Laboratory for Language and Media Processing
Unviersity of Maryland, College Park
and 
Booz Allen Hamilton

All Rights Reserved, 2003
* *************************************************************************/
